#!/bin/ksh 

set -e
set -u
#set -x

CMD=${0##*/}

typeset ATDD_CONFIG_FILE=/etc/rc.config.d/atdd.cfg

typeset driver_name=atdd

typeset mode=666
typeset user=bin
typeset group=bin
typeset action=commands
typeset tape_inst="all"
typeset xrmt_dens="b"
typeset dbg_dens="d"
typeset cfg_dens="c"
typeset ctrl_dens="a"
typeset fileset_xrmt=n
typeset fileset_rmt=n
typeset fileset_dbg=n
typeset fileset_cfg=n
typeset fileset_mchg=n
typeset no_filesets_given=y
typeset device_type=unknown		# can be unknown, drive or changer

# SUBROUTINES {

help ()
{
    exec 1>&2
cat << END_HELP
#
# Usage: $CMD [ args ... ]
# 
# Print commands that will create special files for devices which
# are bound to the "$driver_name" changer driver on an HPUX 11 and 11i
# 
# This command is harmless (unless you pipe the output to a shell).
# 
# Optional arguments:
# 
# Flags:
# 
#   -h                # Print this message.
#
#   -l                # Print only the names of the special files.
#
#   -c                # Print commands to create the special files.
#
#   -ti <changer_inst>   # Restrict output to changer device with instance
#                     # number <changer_inst>.  The default is to show
#                     # output for all instances.  Changer instances can
#                     # be determined with the following command:
#                     #     ioscan -fk -C changer
#
#   -m <mode>         # specify access mode bits (default = $mode)
#   -g <group>	      # specify group ownership  (default = $group)
#   -u <user>	      # specify user  ownership  (default = $user)
#
#   -d <driver>       # specify driver name (default = $driver_name)
#   -f <fileset>      # specify fileset, one of: all, xrmt, dbg, rmt, cfg, mchg.
#
# Examples:
#
#    To show commands that will produce special files for instance 4 of
#    a changer drive bound to the "$driver_name" driver:
#
#        ${CMD} -ti 4
#
#    To see if any special files exist with names that indicate they
#    are linked to the "$driver_name" driver:
#
#        ${CMD} -l
#
#
END_HELP

}



################
# Usage: dec_to_hex dec_value field_width
################
dec_to_hex ()
{
	typeset -i16	hex=10#"$1"
	typeset -R"$2"	out=${hex#*\#}
	echo "${out#*\#}" | sed 's/ /0/g'
}

# this function sets the var OS if it is successful
valid_system ()
{
	uname > /dev/null || return 1;

	case `uname -s` in
		(HP-UX)		;;
		(*)		return 1;;
	esac

	case `uname -m` in
		(9000/[78]*)	;;
		(ia64)	;;
		(*)		return 1;;
	esac

	case `uname -r` in
		(?.11.*)	OS=HP11 ;;
		(*)		return 1;;
	esac
}


valid_tape_instance ()
{
	typeset inst="$1"
	case "$inst" in
		([0-9])			return 0;;
		([0-9][0-9])		return 0;;
		([0-9][0-9][0-9])	return 0;;
		(*)			return 1;;
	esac
}

valid_sticky_tape_instance ()
{
	typeset inst="$1"
	case "$inst" in
		([0-9])			return 0;;
		([0-9][0-9])		return 0;;
		([0-9][0-9][0-9])	return 0;;
	        ([0-9][0-9][0-9][0-9])	return 0;;
	        ([0-9][0-9][0-9][0-9][0-9])	return 0;;
		(*)			return 1;;
	esac
}



valid_major_number ()
{
	typeset maj="$1"
	case "$maj" in
		([0-9]|[0-9][0-9]|[0-9][0-9][0-9]);;
		(*) return 1;;
	esac

	if (( maj > 255 )); then
		return 1
	fi
	return 0

}

valid_target ()
{
	typeset tgt="$1"
	case "$tgt" in
		([0-9]|[0-9][0-9]);;
		(*) return 1;;
	esac
	if (( tgt > 15 )); then
		return 1
	fi
	return 0
}

valid_card_instance ()
{
	typeset inst="$1"
	case "$inst" in
		([0-9])					return 0;;
		([0-9][0-9])			return 0;;
		([0-2][0-9][0-9])		return 0;;
		(*)						return 1;;
	esac
}

valid_hex_card_instance ()
{
	typeset hex="$1"
	case "$hex" in
		([0-9a-f][0-9a-f])	return 0;;
		(*)			return 1;;
	esac
}

valid_hex_target ()
{
	typeset hex_tgt="$1"
	case "$hex_tgt" in
		([0-9a-f]);;
		(*) quit 1 "Unable to determine SCSI target"
	esac
}

valid_lun_target ()
{
	typeset hex_tgt="$1"
	case "$hex_tgt" in
		([1-9a-f]);;
		(*) quit 1 "Invalid LUN."
	esac
}

# usage: quit rc [ -h ] msg msg ...
quit ()
{
	exec 1>&2
	rc=$1; shift
	call_help=no
	[ "$1" = "-h" ] && { call_help=yes; shift; }
	while [ $# -gt 0 ]; do echo "#ERROR: $1"; shift; done
	[ $call_help = yes ] && help
	exit $rc
	return 1
}

parse_fileset ()
{
	no_filesets_given=n
	case "$@" in

		(rmt)	fileset_rmt=y;;

		(xrmt)	fileset_xrmt=y;;

		(dbg)	fileset_dbg=y;;

		(cfg)	fileset_cfg=y;;

		(mchg)  fileset_mchg=y;;

		(all)	fileset_rmt=y
			fileset_xrmt=y
			fileset_dbg=y
			fileset_cfg=y
			fileset_mchg=y
			;;

		(*)	quit 1 "Illegal fileset. Valid values: rmt xrmt dbg cfg mchg"
			;;
	esac
}

verify_device ()
{
	typeset vendor_name=
	typeset vendor_descr=
	typeset perip_type=

	/usr/sbin/ioscan -FkH "$1" | awk -F: 'BEGIN {i=0} {if (i==0) print $18 " " $12; i++}'| \
		while read vendor_name vendor_descr perip_type other; do
	    if [[ "$vendor_name" = "IBM" ]]; then
		if [[ "$perip_type" = "1" ]]; then
			device_type=drive
		elif [[ "$perip_type" = "8" ]]; then
			device_type=changer
		else
			device_type=unknown
		fi
	    else
		device_type=unknown		
	    fi

	done
}

create_sf_params ()
{
	typeset name="$1"
	typeset tape_inst="$2"


	# Global vars used:
        #       ATDD_CONFIG_FILE
	#	xrmt_dens
	#	fileset_xrmt
	#	fileset_rmt
	#	fileset_dbg
	#	fileset_cfg
	#       fileset_mchg
	#typeset	fileset_xrmt=y
	#typeset	fileset_rmt=y
	#typeset	fileset_dbg=y
	#typeset	fileset_cfg=y
	#typeset	fileset_mchg=y

	# Local vars
	typeset ctd=
	typeset maj=
	typeset hwpath=
	typeset ti=
	typeset ci=
	typeset cihex=
	typeset x=
	typeset tgt=
	typeset lun=
	typeset tgthex=
	typeset lunhex=


         /usr/sbin/ioscan -Fk -d "$driver_name" |\
	    awk -F: '{print $7, $8, $11, $13, $19}' |\
	    while read maj minornum hwpath ti ci; do

		if [[ "$tape_inst" != "all" && "$tape_inst" -ne "$ti" ]]; then
			continue
		fi
	
		tgt=$(print $hwpath | awk -F. '{print $(NF-1)}')
		lun=$(print $hwpath | awk -F. '{print $NF}')

		valid_card_instance "$ci" ||
			quit 1 "unable to determine card instance"
		cihex="$(dec_to_hex $ci 2)"
		valid_hex_card_instance "$cihex" ||
			quit 1 "unable to convert card instance to hexadecimal"

		valid_tape_instance "$ti" ||
			quit 1 "unable to determine changer instance"
# JL lets not quit in this case since it validates every IBM device
# some of which may not be bound properly to atdd (in the case of NO_HW) or disk drives	
# proper way to validate the major number is to check the lsdev and ioscan outputs ..
		valid_major_number "$maj" || echo "unable to determine maj number" >&2|| continue
	
		valid_target "$tgt" ||
			quit 1 "unable to determine SCSI target"
		tgthex="$(dec_to_hex $tgt 1)"
		valid_hex_target "$tgthex" ||
			quit 1 "unable to convert SCSI target to hexadecimal"

		if [[ $lun -gt 0 ]]; then
		valid_target "$lun" ||
			quit 1 "unable to determine SCSI lun"
		lunhex="$(dec_to_hex $lun 1)"
		valid_lun_target "$lunhex" ||
			quit 1 "unable to convert SCSI lun to hexadecimal"
		fi

	
		ctd="c${ci}t${tgt}d${lun}"
	
                #sets the device_type, vendor name, etc
		verify_device "$hwpath" 






#		if [[ $device_type = changer && $lun -eq 0 ]]; then
#	                echo /dev/rmt/${ti}chng       $maj 0x${cihex}${tgthex}000
#		elif [[ $device_type != drive && $lun -ne 0 ]]; then
#			echo /dev/rmt/${ti}chng    $maj 0x${cihex}${tgthex}${lunhex}00
#		else
		  if [[ $fileset_xrmt = y ]]; then
                             echo /dev/rmt/${ti}x $maj 0x${cihex}${tgthex}00${xrmt_dens}
		  fi

		  if [[ $fileset_dbg = y ]]; then
	                     echo /dev/rmt/atdd.dbg $maj 0xFFFFF${dbg_dens}
			     typeset fileset_dbg=n
		  fi

		  if [[ $fileset_cfg = y ]]; then
	                     echo /dev/rmt/atdd.cfg $maj 0xFFFFF${cfg_dens}
			     typeset fileset_cfg=n
			     echo /dev/rmt/atdd.ctrl $maj 0xFFFFF${ctrl_dens}
		  fi




		  if [[ $fileset_rmt = y && $lun -eq 0 ]]; then
# + 11 opens the device in raw mode.
 		             /opt/atdd/bin/atdd_specialfile ${maj} $(($minornum+11)) 1 

		             # instance based
	                     echo /dev/rmt/${ti}m       $maj 0x${cihex}${tgthex}000
        	             echo /dev/rmt/${ti}mb      $maj 0x${cihex}${tgthex}080
                	     echo /dev/rmt/${ti}mn      $maj 0x${cihex}${tgthex}040
                             echo /dev/rmt/${ti}mnb     $maj 0x${cihex}${tgthex}0c0
                             echo /dev/rmt/${ti}mnr     $maj 0x${cihex}${tgthex}04b
			     # now the stape card target device 
	                     echo /dev/rmt/${ctd}BEST   $maj 0x${cihex}${tgthex}000
        	             echo /dev/rmt/${ctd}BESTb  $maj 0x${cihex}${tgthex}080
                	     echo /dev/rmt/${ctd}BESTn  $maj 0x${cihex}${tgthex}040
                             echo /dev/rmt/${ctd}BESTnb $maj 0x${cihex}${tgthex}0c0

		  fi
		  if [[ $device_type = drive && $lun -ne 0 ]]; then
 		             /opt/atdd/bin/atdd_specialfile ${maj} $(($minornum+11)) 1 
	                     echo /dev/rmt/${ti}m       $maj 0x${cihex}${tgthex}${lunhex}00
        	             echo /dev/rmt/${ti}mb      $maj 0x${cihex}${tgthex}${lunhex}80
                	     echo /dev/rmt/${ti}mn      $maj 0x${cihex}${tgthex}${lunhex}40
                             echo /dev/rmt/${ti}mnb     $maj 0x${cihex}${tgthex}${lunhex}c0
                             echo /dev/rmt/${ti}mnr     $maj 0x${cihex}${tgthex}${lunhex}4b

	                     echo /dev/rmt/${ctd}BEST   $maj 0x${cihex}${tgthex}${lunhex}00
        	             echo /dev/rmt/${ctd}BESTb  $maj 0x${cihex}${tgthex}${lunhex}80
                	     echo /dev/rmt/${ctd}BESTn  $maj 0x${cihex}${tgthex}${lunhex}40
                             echo /dev/rmt/${ctd}BESTnb $maj 0x${cihex}${tgthex}${lunhex}c0
		  fi
#		fi
	    done

}


# SUBROUTINES }
lsdev  |grep "$driver_name" |cut -c 7-10 | read drvmjr
#echo $drvmjr
ls -Fal /dev/rmt/* | cut -c 38-40,63-80 |grep ^"$drvmjr"|xargs -n 1 rm -f



# first reset specialfile counts
/opt/atdd/bin/atdd_specialfile -c


# Parse command line args
while [ $# -gt 0 ]; do
	case "$1" in
		(-h)	quit 0 -h
			;;
		(-ti)	[ $# -ge 2 ] || quit 1 "Expected arg after -ti" "Use -h for help"
			tape_inst="$2"
			valid_tape_instance "$tape_inst" ||
				quit 1 "Invalid Changer instance value" "Use -h for help"
			shift 2
			;;
		(-m)	[ $# -ge 2 ] || quit 1 "Expected arg after -m" "Use -h for help"
			mode="$2"
			shift 2
			;;
		(-g)	[ $# -ge 2 ] || quit 1 "Expected arg after -g" "Use -h for help"
			group="$2"
			shift 2
			;;
		(-u)	[ $# -ge 2 ] || quit 1 "Expected arg after -u" "Use -h for help"
			user="$2"
			shift 2
			;;
		(-d)	[ $# -ge 2 ] || quit 1 "Expected arg after -d" "Use -h for help"
			driver_name="$2"
			shift 2
			;;
		(-f)	[ $# -ge 2 ] || quit 1 "Expected arg after -f" "Use -h for help"
			parse_fileset "$2"
			shift 2
			;;
		(-l)	action=list
			shift
			;;
		(-r)	action=remove
			shift
			;;
		(-c)	action=commands
			shift
			;;
		(*)	quit 1 "Bad arg: $1" "Use -h for help"
			;;
	esac
done


valid_system || 
	quit 1 "This command only works on HP-UX 11 and 11i, Series 800 Systems." \
		"Use -h for help"

typeset new_files=
typeset sfname=
typeset major=
typeset minor=

if [[ $no_filesets_given = y ]]; then
	parse_fileset rmt
	parse_fileset dbg
	parse_fileset cfg
	parse_fileset mchg
fi

typeset first=yes

create_sf_params "$driver_name" "$tape_inst" |
  while read sfname major minor; do
      if [[ "$first" = "yes" && "$action" = "commands" ]]; then
          first=no
          echo set -u
#          echo set -x #print or don't print the commands as they are executed!
		  echo typeset drive_changer=
		  mkdir -p /dev/rmt/
           echo ""
      fi

      case "$action" in

          (list)          echo "$sfname"
                          ;;

          (commands)      if [[ -c $sfname ]]; then
                            echo "rm -f $sfname"
                          fi
						echo "mknod $sfname c $major $minor"
						new_files="$new_files $sfname"
						  
#						  test "`echo $sfname |grep \".*mn$\"`" && \
#{    
#							echo /opt/atdd/bin/atdd_util $sfname
#							echo if [[ \$\? -eq 3 ]]\; then
#								echo $sfname | sed s/mn/chng/ | read sfname_new
#								echo rm -f $sfname_new
#								echo mknod $sfname_new c $major $minor
#								echo drive_changer=\"\$drive_changer $sfname_new\"
#							echo fi
#}
						echo ""
                          ;;
          (remove)        if [[ -c $sfname ]]; then
                              echo "rmsf $sfname"
                          fi
                          ;;
          (*)             quit 1 "There is a bug in this script."
                          ;;

      esac

  done

if [[ "$action" = "commands" && "$new_files" != "" ]]; then
    echo ""
    echo "chmod $mode  $new_files \$drive_changer"
    echo ""
    echo "chown $user  $new_files \$drive_changer"
    echo ""
    echo "chgrp $group $new_files \$drive_changer"
fi

exit 0











